/**  
 * @file csl_serdes_10ge.h
 *
 * @brief 
 *  Header file for functional layer of CSL SERDES. 
 *
 *  It contains the various enumerations, structure definitions and function 
 *  declarations
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2013, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
  

/** ============================================================================ 
 *
 * @defgroup CSL_SERDES_10GE SERDES 10GE
 * @ingroup CSL_SERDES_API
 *
 * @section Introduction
 *
 * @subsection xxx Overview
 * This module deals with setting up SERDES configuration for 10GE. The API flow should be as follows:   \n
 *
 * CSL_10GeSerdesInit (base_addr, ref_clock, rate);
 *
 * CSL_10GeSerdesLaneConfig (base_addr, ref_clock, rate, lane_num);
 *
 * CSL_10GeSerdesComEnable (base_addr, rate);
 *
 * CSL_SerdesTXRXSetEqualizer(base_addr, num_lanes, phy_type, set_eq_flags);
 *
 * CSL_10GeSerdesPllEnable (base_addr, link_rate);
 *
 * CSL_10GeSerdesLaneEnable (base_addr, lane_num, link_rate, loopback_mode);
 *
 * CSL_10GeSerdesGetStatus (base_addr, num_lanes);
 *
 * CSL_Serdes_DFE_OffsetCalibration(base_addr, num_lanes, phy_type, tx_coeff);
 *
 * CSL_10GeSerdesEnableXGMIIPort (base_addr);
 * @subsection References
 *    
 * ============================================================================
 */ 
#include <ti/csl/csl_serdes.h>
#include <ti/csl/src/ip/serdes_sb/V0/csl_wiz8_sb_PhyB.h>





/** @addtogroup CSL_SERDES_10GE
 @{ */
/** ============================================================================
 *   @n@b CSL_10GeSerdesInit
 *
 *   @b Description
 *   @n This API initializes the Serdes CMU and COMLANE registers.
 *
 *   @b Arguments   base_addr, ref_clock, rate
 *
 *   <b> Return Value  </b>  CSL_SERDES_RESULT
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim

    csl_retval = CSL_10GeSerdesInit(CSL_XGE_SERDES_CFG_REGS, CSL_SERDES_REF_CLOCK_156p25M, CSL_SERDES_LINK_RATE_10p3125G);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE CSL_SERDES_RESULT CSL_10GeSerdesInit
(
        uint32_t  base_addr,
        CSL_SERDES_REF_CLOCK  ref_clock,
        CSL_SERDES_LINK_RATE  rate
)
{
    if (ref_clock != CSL_SERDES_REF_CLOCK_156p25M)
    {
        return CSL_SERDES_INVALID_REF_CLOCK;
    }

    /* Disable PLL */
    /* PLL2 disable */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fc0 + 0x34), 31, 29, (uint32_t)0x4);
    /* PLL1 disable */ 
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fc0 + 0x34), 27, 25, (uint32_t)0x4); 

    if (ref_clock == CSL_SERDES_REF_CLOCK_156p25M && rate == CSL_SERDES_LINK_RATE_10p3125G)
    {


        csl_wiz8_sb_PhyB_1p25G_156p25MHz_cmu0(base_addr);
        csl_wiz8_sb_PhyB_10p3125G_156p25MHz_cmu1(base_addr);
    }

    if (ref_clock == CSL_SERDES_REF_CLOCK_156p25M && rate == CSL_SERDES_LINK_RATE_1p25G)
    {
        csl_wiz8_sb_PhyB_1p25G_156p25MHz_cmu0(base_addr);
        csl_wiz8_sb_PhyB_10p3125G_156p25MHz_cmu1(base_addr);
    }

    if (!(rate == CSL_SERDES_LINK_RATE_10p3125G || rate == CSL_SERDES_LINK_RATE_1p25G))
    {
        return CSL_SERDES_INVALID_LANE_RATE;
    }

    return CSL_SERDES_NO_ERR;
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesLaneConfig
 *
 *   @b Description
 *   @n This API configures the Serdes Lanes
 *
 *   @b Arguments   base_addr, ref_clock, rate, lane_num
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_EthernetSerdesLaneConfig(CSL_XGE_SERDES_CFG_REGS, CSL_SERDES_REF_CLOCK_156p25M, CSL_SERDES_LINK_RATE_10p3125G, 1);

    @endverbatim
 * ===========================================================================
 */        
CSL_IDEF_INLINE void CSL_10GeSerdesLaneConfig
(
        uint32_t base_addr,
        CSL_SERDES_REF_CLOCK ref_clock,
        CSL_SERDES_LINK_RATE rate,
        uint32_t lane_num
)
{
    if (ref_clock == CSL_SERDES_REF_CLOCK_156p25M && rate == CSL_SERDES_LINK_RATE_10p3125G)
    {
        if (lane_num == 0)
        {
            csl_wiz8_sb_PhyB_10p3125G_16bit_lane1(base_addr);

            /* Disable Auto Negotiation */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x180),4,4, 0x0);

            /* Disable Link Training */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x1C0),9,9, 0x0);
        }
        else if (lane_num == 1)
        {
            csl_wiz8_sb_PhyB_10p3125G_16bit_lane2(base_addr);

            /* Disable Auto Negotiation */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x180),4,4, 0x0);

            /* Disable Link Training */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x1C0),9,9, 0x0);
        }       

    }

    if (ref_clock == CSL_SERDES_REF_CLOCK_156p25M && rate == CSL_SERDES_LINK_RATE_1p25G)
    {
        if (lane_num == 0)
        {
            csl_wiz8_sb_PhyB_aneg_lane1(base_addr);

                    /* Disable Auto Negotiation */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x180),4,4, 0x0);         

        }
        else if (lane_num == 1)
        {
            csl_wiz8_sb_PhyB_aneg_lane2(base_addr);

            /* Disable Auto Negotiation */
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x180),4,4, 0x0);         
        }
    }
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesComEnable
 *
 *   @b Description
 *   @n This API enables the Serdes CMU and COMLANE
 *
 *   @b Arguments   base_addr, rate
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesComEnable(CSL_XGE_SERDES_CFG_REGS, CSL_SERDES_LINK_RATE_10p3125G);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_10GeSerdesComEnable
(
        uint32_t base_addr,
        CSL_SERDES_LINK_RATE rate
)
{
    /* Configure COMLANE */
    if(rate == CSL_SERDES_LINK_RATE_10p3125G)
    {
        csl_wiz8_sb_PhyB_10p3125G_comlane(base_addr);
    }
    
    if(rate == CSL_SERDES_LINK_RATE_1p25G)
    {
        csl_wiz8_sb_PhyB_1p25G_156p25MHz_comlane(base_addr);
    }
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesLaneEnable
 *
 *   @b Description
 *   @n This API enables the Serdes CMU and COMLANE
 *
 *   @b Arguments   base_addr, lane_num, rate, loopback_mode
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesLaneEnable(CSL_XGE_SERDES_CFG_REGS, 0, CSL_SERDES_LINK_RATE_10p3125G, CSL_SERDES_LOOPBACK_ENABLED);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_10GeSerdesLaneEnable
(
        uint32_t base_addr,
        uint32_t lane_num,
        CSL_SERDES_LINK_RATE rate,
        CSL_SERDES_LOOPBACK loopback_mode
)
{

    /* Set Lane Control Rate */
    if (rate == CSL_SERDES_LINK_RATE_10p3125G)
    {
        CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),28,26, 0x04);
        CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),12,10, 0x04);
    }

    if (rate == CSL_SERDES_LINK_RATE_1p25G)
    {
        *(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num) = 0xf800f8c0;
    }

    /* Set Bus-Width */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),23,21, 0x7);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num), 5, 3, 0x7);
    
    /* Enable PCS Overlay and Lane Select 10GKR */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),16,16, 0x1);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),19,19, 0x1);
    
    /* Enable Lanes */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),31,29, (uint32_t)0x7);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),15,13, 0x7);


    /* Set NES bit if Loopback Enabled */
    if (loopback_mode == CSL_SERDES_LOOPBACK_ENABLED)
    {
        if (rate == CSL_SERDES_LINK_RATE_10p3125G)
        {
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1)),7,0, 0x04);
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x04),2,1,3);
        }

        else
        {
            CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1)),31,24, 0x40);
        }
    }
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesLaneDisable
 *
 *   @b Description
 *   @n This API disables the Serdes Lanes
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesLaneDisable(CSL_XGE_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_10GeSerdesLaneDisable
(
        uint32_t base_addr,
        uint32_t lane_num
)
{
    /* Puts Lane # in Low Power Mode */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),30,29, 0x0);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),14,13, 0x0);
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesLaneReset
 *
 *   @b Description
 *   @n This API resets the Serdes Lanes
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesLaneReset(CSL_XGE_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_10GeSerdesLaneReset
(
        uint32_t base_addr,
        uint32_t lane_num
)
{
    /* Reset RX PLL */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x28),29,29, 0x1);
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesPllEnable
 *
 *   @b Description
 *   @n This API enables the PLL
 *
 *   @b Arguments   base_addr, rate
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesPllEnable(CSL_XGE_SERDES_CFG_REGS, CSL_SERDES_LINK_RATE_10p3125G);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_10GeSerdesPllEnable
(
        uint32_t base_addr,
        CSL_SERDES_LINK_RATE rate
)
{
    /* PHY-B Reset Clear */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0xa00), 5, 0, 0x1F );

    /* Set PLL Enable Val */
    if (rate == CSL_SERDES_LINK_RATE_1p25G)
        *(volatile uint32_t *)(base_addr + 0x1ff4) = 0xe0000000;

    if (rate == CSL_SERDES_LINK_RATE_10p3125G)
    {
        CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1ff4), 31, 29, (uint32_t)0x7 );
        CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1ff4), 27, 25, 0x7 );
    }
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesTXBClkMode
 *
 *   @b Description
 *   @n This API sets PHY-B Clock Mode to aid accurate time stamping of 1588. It is an optional function call for application requiring higher clock synchronization accuracy.
 *
 *   @b Arguments   base_addr, rate
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesTXBClkMode(CSL_XGE_SERDES_CFG_REGS, CSL_SERDES_LINK_RATE_10p3125G);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_10GeSerdesTXBClkMode
(
        uint32_t base_addr,
        CSL_SERDES_LINK_RATE rate
)
{
    if (rate == CSL_SERDES_LINK_RATE_1p25G)
    {
        *(volatile uint32_t *)(base_addr + 0x1fd0) = 0x20000000;
        *(volatile uint32_t *)(base_addr + 0x1ff4) = 0x00380000;
        *(volatile uint32_t *)(base_addr + 0x1fd0) = 0x00000000;
    }
    else
    {
        return;
    }
}


/** ============================================================================
 *   @n@b CSL_10GeSerdesGetStatus
 *
 *   @b Description
 *   @n This API returns the status of the PLL lock
 *
 *   @b Arguments   base_addr, num_lanes
 *
 *   <b> Return Value  </b>  CSL_SERDES_STATUS
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesGetStatus(CSL_XGE_SERDES_CFG_REGS, 2);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE CSL_SERDES_STATUS CSL_10GeSerdesGetStatus
(
        uint32_t base_addr,
        uint32_t num_lanes
)
{
    uint32_t i;
    CSL_SERDES_STATUS retval = CSL_SERDES_STATUS_PLL_LOCKED;

    /* Read Lane OK Status Bits */
    for (i=0; i < num_lanes; i++)
    {
        retval &= (CSL_SERDES_STATUS)CSL_FEXTR(*(volatile uint32_t *)
                  (base_addr + 0xa00 + 0x1f8), (29+i), (29+i));
    }

    return retval;
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesGetLaneStatus
 *
 *   @b Description
 *   @n This API returns the specific lane status for the PLL lock
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>  CSL_SERDES_STATUS
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesGetLaneStatus(CSL_XGE_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE CSL_SERDES_STATUS CSL_10GeSerdesGetLaneStatus
(
        uint32_t base_addr,
        uint32_t lane_num
)
{
	CSL_SERDES_STATUS retval = CSL_SERDES_STATUS_PLL_NOT_LOCKED;

    /* Read Lane OK Status Bits */
    retval = (CSL_SERDES_STATUS)CSL_FEXTR(*(volatile uint32_t *)
                 (base_addr + 0xa00 + 0x1f8), (29+lane_num), (29+lane_num));
    
    return retval;
}

/** ============================================================================
 *   @n@b CSL_10GeSerdesEnableXGMIIPort
 *
 *   @b Description
 *   @n This API enables the XGMII Ports
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n XGMII rate will be set based on SERDES setup instead of manual mode. This sets the Submodule Control Register bits 0 and 1.
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesEnableXGMIIPort(CSL_XGE_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_10GeSerdesEnableXGMIIPort
(
        uint32_t base_addr
)
{   
    *(volatile uint32_t *)(base_addr + 0x0c) = 0x3;
}



/** ============================================================================
 *   @n@b CSL_10GeSerdesShutdown
 *
 *   @b Description
 *   @n This API shuts down the Serdes by Disabling and Resetting the Serdes
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim

     CSL_10GeSerdesShutdown(CSL_XGE_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_10GeSerdesShutdown
(
        uint32_t base_addr
) 
{
    uint32_t i=0;

    /* Disables and Resets SERDES for each lane */
    for(i=0; i<2; i++)
    {
        CSL_10GeSerdesLaneDisable(base_addr, i);
        CSL_10GeSerdesLaneReset(base_addr, i);
    }

    /* Disable CMU TX PLL */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1FC0 + 0x34), 30, 29, 0x0);

    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1FC0 + 0x34), 26, 25, 0x0);

    /* Bit 28 of CMU_10 Toggled. Reset TX PLL for all lanes */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x10), 28, 28, 0x1);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x0c00 + 0x10), 28, 28, 0x1);
}

/* @} */
