/**
 * @file csl_serdes_iqn.h
 *
 * @brief
 *  Header file for functional layer of CSL SERDES IQN.
 *
 *  It contains the various enumerations, structure definitions and function
 *  declarations
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2013, Texas Instruments, Inc.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** ============================================================================
 *
 * @defgroup CSL_SERDES_IQN SERDES IQN
 * @ingroup CSL_SERDES_API
 *
 * @section Introduction
 *
 * @subsection xxx Overview
 * This module deals with setting up SERDES configuration for IQN. The API flow should be as follows:   \n
 *
 * CSL_IQNSerdesInit (base_addr, ref_clock, rate);
 *
 * CSL_IQNSerdesLaneEnable (base_addr, lane_num, link_rate);
 *
 * CSL_IQNSerdesPllEnable (base_addr);
 *
 * CSL_IQNSerdesGetStatus (base_addr);
 *
 * @subsection References
 *
 * ============================================================================
 */

#include <ti/csl/csl_serdes.h>
#include <ti/csl/src/ip/serdes_sb/V0/csl_wiz8_sb.h>


/** @addtogroup CSL_SERDES_IQN
 @{ */
/** ============================================================================
 *   @n@b CSL_IQNSerdesInit
 *
 *   @b Description
 *   @n This API initializes the Serdes CMU and COMLANE registersL.
 *
 *   @b Arguments   base_addr, ref_clock, rate
 *
 *   <b> Return Value  </b>  CSL_SERDES_RESULT
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
    csl_retval = CSL_IQNSerdesInit(CSL_CSISC2_0_SERDES_CFG_REGS, CSL_SERDES_REF_CLOCK_122p88M, CSL_SERDES_LINK_RATE_6p144G);

    @endverbatim
 * ===========================================================================
 */       
CSL_IDEF_INLINE CSL_SERDES_RESULT CSL_IQNSerdesInit
(
 uint32_t  base_addr,
 CSL_SERDES_REF_CLOCK  ref_clock,
 CSL_SERDES_LINK_RATE  rate
)
{
    if (!(ref_clock == CSL_SERDES_REF_CLOCK_122p88M || ref_clock == CSL_SERDES_REF_CLOCK_153p6M))
    {
        return CSL_SERDES_INVALID_REF_CLOCK;
    } 


    if (ref_clock == CSL_SERDES_REF_CLOCK_122p88M && rate == CSL_SERDES_LINK_RATE_4p9152G)
    {   
        csl_wiz8_sb_refclk122p88MHz_20bit_4p9152Gbps_2l1c(base_addr);
    }
    
    else if (ref_clock == CSL_SERDES_REF_CLOCK_122p88M && rate == CSL_SERDES_LINK_RATE_6p144G)
    {
        csl_wiz8_sb_refclk122p88MHz_20bit_6p144Gbps_2l1c(base_addr);
    }
    
    else if (ref_clock == CSL_SERDES_REF_CLOCK_122p88M && rate == CSL_SERDES_LINK_RATE_9p8304G)
    {
        csl_wiz8_sb_refclk122p88MHz_20bit_9p8304Gbps_2l1c_AIF_sr1(base_addr);
    } 

    else if (ref_clock == CSL_SERDES_REF_CLOCK_153p6M && rate == CSL_SERDES_LINK_RATE_6p144G)
    {
        csl_wiz8_sb_refclk153p6MHz_20bit_6p144Gbps_2l1c(base_addr);
    }

    else if (ref_clock == CSL_SERDES_REF_CLOCK_153p6M && rate == CSL_SERDES_LINK_RATE_4p9152G)
    {
        csl_wiz8_sb_refclk153p6MHz_20bit_4p9152Gbps_2l1c_sr1(base_addr);
    }   

    if (!(rate == CSL_SERDES_LINK_RATE_4p9152G || CSL_SERDES_LINK_RATE_6p144G || CSL_SERDES_LINK_RATE_9p8304G))
    {
        return CSL_SERDES_INVALID_LANE_RATE;
    }
    
    return CSL_SERDES_NO_ERR;
}


/** ============================================================================
 *   @n@b CSL_IQNSerdesLaneEnable
 *
 *   @b Description
 *   @n This API enables the Serdes CMU and COMLANE
 *
 *   @b Arguments   base_addr, lane_num, link_rate
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesLaneEnable(CSL_CSISC2_0_SERDES_CFG_REGS, 0, CSL_SERDES_LOOPBACK_DISABLED, CSL_SERDES_LANE_FULL_RATE);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_IQNSerdesLaneEnable
(
 uint32_t base_addr,
 uint32_t lane_num,
 CSL_SERDES_LOOPBACK loopback_mode,
 CSL_SERDES_LANE_CTRL_RATE lane_ctrl_rate
) 
{
     /* Bit 28 Toggled. Bring it out of Reset TX PLL for all lanes */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1) + 0x28),29,29, 0x0);
    
    /* Set Lane Control Rate */
    if(lane_ctrl_rate == CSL_SERDES_LANE_FULL_RATE)
    {
        /* RX Full Rate. TX is always Full Rate */
    	*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num) = 0xF000F000;
    }
    
    else if(lane_ctrl_rate == CSL_SERDES_LANE_HALF_RATE)
    {
        /* RX Half Rate. TX is always Full Rate */        
    	*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num) = 0xF000F400;
    }
    
    else if(lane_ctrl_rate == CSL_SERDES_LANE_QUARTER_RATE)
    {
	/* RX Quarter Rate. TX is always Full Rate */        
    	*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num) = 0xF000F800;
    }

    /* Set NES bit if Loopback Enabled */
    if (loopback_mode == CSL_SERDES_LOOPBACK_ENABLED)
    {
        CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x200*(lane_num+1)),31,24, 0x40);
    }

}


/** ============================================================================
 *   @n@b CSL_IQNSerdesPllEnable
 *
 *   @b Description
 *   @n This API enables the IQN Serdes PLL 
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesPllEnable(CSL_CSISC2_0_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_IQNSerdesPllEnable
(
 uint32_t base_addr
)
{
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1FC0 + 0x34), 31, 29, (uint32_t)0x7);
}


/** ============================================================================
 *   @n@b CSL_IQNSerdesLaneDisable
 *
 *   @b Description
 *   @n This API disables the Serdes Lanes
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesLaneDisable(CSL_CSISC2_0_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_IQNSerdesLaneDisable
(
 uint32_t base_addr,
 uint32_t lane_num
)
{
    /* Puts Lane # in Low Power Mode */
    /*Bit 31 and Bit 15 needs to be set for PLL Overlay override */     
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),31,29, (uint32_t)0x4);
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1fe0 + 4*lane_num),15,13, (uint32_t)0x4);
}


/** ============================================================================
 *   @n@b CSL_IQNSerdesGetStatus
 *
 *   @b Description
 *   @n This API returns the status of the Serdes Lane PLL lock
 *
 *   @b Arguments   base_addr, num_lanes
 *
 *   <b> Return Value  </b>  CSL_SERDES_STATUS
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesGetStatus(CSL_CSISC2_0_SERDES_CFG_REGS, 2);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE CSL_SERDES_STATUS CSL_IQNSerdesGetStatus
(
 uint32_t base_addr,
 uint32_t num_lanes
)
{
    uint32_t i;
    CSL_SERDES_STATUS retval = CSL_SERDES_STATUS_PLL_LOCKED;

    /* Check LANEx_PLL_LOCK bit in LANE_PLL_STS register */
    for (i=0; i < num_lanes; i++)
    {
        retval &= (CSL_SERDES_STATUS)CSL_FEXTR(*(volatile uint32_t *)(base_addr + 0x1fd4), (3 + 4*i), (3 + 4*i));
    }

    return retval;
}

/** ============================================================================
 *   @n@b CSL_IQNSerdesGetLaneStatus
 *
 *   @b Description
 *   @n This API returns the lane specific status of the Serdes Lane PLL lock
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>  CSL_SERDES_STATUS
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesGetLaneStatus(CSL_CSISC2_0_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE CSL_SERDES_STATUS CSL_IQNSerdesGetLaneStatus
(
 uint32_t base_addr,
 uint32_t lane_num
)
{
    CSL_SERDES_STATUS retval = CSL_SERDES_STATUS_PLL_LOCKED;

    /* Check LANEx_PLL_LOCK bit in LANE_PLL_STS register */
    retval = (CSL_SERDES_STATUS)CSL_FEXTR(*(volatile uint32_t *)(base_addr + 0x1fd4), (3 + 4*lane_num), (3 + 4*lane_num));

    return retval;
}


/** ============================================================================
 *   @n@b CSL_IQNSerdesShutdown
 *
 *   @b Description
 *   @n This API shuts down the Serdes by Disabling and Resetting the Serdes
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 
     CSL_IQNSerdesShutdown(CSL_CSISC2_0_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_IQNSerdesShutdown
(
 uint32_t base_addr
) 
{
    uint32_t i=0;

    /* Disables and Resets SERDES for each lane */
    for(i=0; i<4; i++)
    {
        CSL_IQNSerdesLaneDisable(base_addr, i);
    } 
    
    /* Disable CMU TX PLL (Resets CMU) */
    CSL_FINSR(*(volatile uint32_t *)(base_addr + 0x1FC0 + 0x34), 31, 29, (uint32_t)0x4);

}

/** ============================================================================
 *   @n@b CSL_IQN2SerdesIsReset
 *
 *   @b Description
 *   @n This API returns a flag indicating whether the SerDes is in reset (1), or
 *   whether a previous configuration was applied to the SerDes
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None
 *
 *  <b> Usage Constraints: </b>
 *  @n None
 *
 *   @b Example
 *   @verbatim
 	 uint32_t serdesIsReset;
     serdesIsReset = CSL_IQN2SerdesIsReset(CSL_CSISC2_0_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE uint32_t CSL_IQN2SerdesIsReset
(
 uint32_t base_addr
)

{
	uint32_t serdesIsReset;
	serdesIsReset = (CSL_FEXTR(*(volatile uint32_t *)(base_addr + 0xa00), 4, 0) == 0);
	return (serdesIsReset);
}

/* @} */

